#define MAXPUFFER 200

class RS232
{
  private:
    int i;
	unsigned long hilf;
	unsigned int k;
	char puffer[MAXPUFFER];
  public:
    RS232(uint32_t baudrate=115200)
    {
  	    //setzen der Baudrate:
	    UBRRH = (uint8_t) ((TAKTFREQUENZ/8/baudrate-1)>>8); 
	    UBRRL = (uint8_t)  (TAKTFREQUENZ/8/baudrate-1);     
        //einfache (nicht doppelte) bertragungsgeschwindigkeit:
        UCSRA = (1<<U2X);
	    //Setzen des Nachrichtenformats: 8 Datenbits, 1 Stopbits
	    UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
    }
    void start(bool senden, bool empfangen)
    {
        if(senden==true)
            UCSRB |= 1<<TXEN;
        if(empfangen==true)
            UCSRB |= 1<<RXEN;
    }
    void stop()
    {
            UCSRB &= 255 - (1<<TXEN) - (1<<RXEN);
    }

    char* holeText()
    {
			i=0;
            while(true)
			{
	            while( !(UCSRA & (1<<RXC)));
				puffer[i]=UDR;
                //Zeichen zurckschicken:
                    while( !(UCSRA & (1<<UDRE)) ); 
                    UDR = puffer[i];       
                //Ende Zeichen zurckschicken              
                i++;
				if(puffer[i-1]=='\r' || i>=MAXPUFFER) //Escape-Taste
				    break;
	        }
            puffer[i]='\0';

			return puffer;
    }


    void sendeZeichen(uint8_t zeichen)
    {
        while( !(UCSRA & (1<<UDRE)) ); 
        UDR = zeichen;                 
    }


    void sendeZahl(uint8_t zahl)
    {
      if(zahl>100)
        sendeZeichen( 48+zahl/100 );
      else
        sendeZeichen( ' ' );

      if(zahl>10)
        sendeZeichen( 48+(zahl % 100)/10 );
      else
        sendeZeichen( ' ' );

        sendeZeichen( 48+zahl%10 );
        sendeZeichen(' ');
    }

    void sendeZeilenumbruch()
    {
        sendeZeichen('\n');
        sendeZeichen('\r');
    }

    void sendeText(char* text)
    {
	    i=0;
	    while(text[i]!='\0')
		    sendeZeichen(text[i++]);
    }


        void sendeDoubleZahl(double zahl) //achtung: Bereich +/-4200 bei 6 Nachkommastellen
		{            
            if(zahl<0.0)
			{
			    k=1;
				zahl = -zahl;
            }
			else
			{
			    k=0;
			}

            if(zahl>4200.0)
			{
			    sendeText("zu gross! ");
				return;
			}
            //-4200.000000
			//012345678901
            puffer[0] = ' ';
            puffer[1] = ' ';
            puffer[2] = ' ';
            puffer[3] = ' ';
            puffer[4] = ' ';
            puffer[5] = ' ';
            puffer[6] = ' ';
            puffer[7] = ' ';
            puffer[8] = ' ';
            puffer[9] = ' ';
            puffer[10] = ' ';
            puffer[11] = ' ';
            puffer[12] = '\0';

            hilf = (unsigned long)(1000000.0*zahl);

		    i=11;
			while(hilf!=0 || i>=4)
			{
			    puffer[i] = (hilf%10) + 48;
				hilf/=10;
				i--;
				if(i==5)
				{
				    puffer[i]='.';
					i--;
				}
			}
            if(k==1)
			    puffer[i] = '-';
			sendeText(puffer);
			sendeText(" ");
		}

        void sendeUnsignedIntegerZahl(unsigned int zahl)
		{
            if(zahl==0)
			{
			    sendeText("     0 ");
				return;
			}

		    puffer[0] = ' ';
		    puffer[1] = ' ';
		    puffer[2] = ' ';
		    puffer[3] = ' ';
		    puffer[4] = ' ';
		    puffer[5] = '\0';
 
		    i=4;
			while(zahl!=0)
			{
			    puffer[i] = (zahl%10) + 48;
				zahl/=10;
				i--;
			}
			sendeText(puffer);
			sendeZeichen(' ');
		}
};
